@@ -8,7 +8,7 @@ from sqlfluff.api import lint, fix, parse, list_rules, list_dialects
 # Import metadata (using importlib_metadata backport for python versions <3.8)
 if sys.version_info >= (3, 8):
     from importlib import metadata
-else:
+else:  # pragma: no cover
     import importlib_metadata as metadata
 
 __all__ = (
@@ -33,4 +33,4 @@ elif sys.version_info[1] < 7:
     )
 
 # Register helper functions to support variable introspection on failure.
-pytest.register_assert_rewrite("sqlfluff.testing")
+pytest.register_assert_rewrite("sqlfluff.utils.testing")
