@@ -13,10 +13,6 @@ import yaml
 
 import click
 
-# For the profiler
-import pstats
-from io import StringIO
-
 # To enable colour cross platform
 import colorama
 from tqdm import tqdm
@@ -721,7 +717,7 @@ def lint(
     if not nofail:
         if not non_human_output:
             formatter.completion_message()
-        sys.exit(result.stats()["exit code"])
+        sys.exit(result.stats(EXIT_FAIL, EXIT_SUCCESS)["exit code"])
     else:
         sys.exit(EXIT_SUCCESS)
 
@@ -1145,9 +1141,6 @@ def quoted_presenter(dumper, data):
 @common_options
 @core_options
 @click.argument("path", nargs=1, type=click.Path(allow_dash=True))
-@click.option(
-    "--recurse", default=0, help="The depth to recursively parse to (0 for unlimited)"
-)
 @click.option(
     "-c",
     "--code-only",
@@ -1186,9 +1179,6 @@ def quoted_presenter(dumper, data):
         "stdout logging."
     ),
 )
-@click.option(
-    "--profiler", is_flag=True, help="Set this flag to engage the python profiler."
-)
 @click.option(
     "--parse-statistics",
     is_flag=True,
@@ -1211,7 +1201,6 @@ def parse(
     include_meta: bool,
     format: str,
     write_output: Optional[str],
-    profiler: bool,
     bench: bool,
     nofail: bool,
     logger: Optional[logging.Logger] = None,
@@ -1236,7 +1225,6 @@ def parse(
     output_stream = make_output_stream(c, format, write_output)
     lnt, formatter = get_linter_and_formatter(c, output_stream)
     verbose = c.get("verbose")
-    recurse = c.get("recurse")
 
     progress_bar_configuration.disable_progress_bar = True
 
@@ -1250,18 +1238,6 @@ def parse(
         stderr_output=non_human_output,
     )
 
-    # TODO: do this better
-
-    if profiler:
-        # Set up the profiler if required
-        try:
-            import cProfile
-        except ImportError:  # pragma: no cover
-            click.echo("The cProfiler is not available on your platform.")
-            sys.exit(EXIT_ERROR)
-        pr = cProfile.Profile()
-        pr.enable()
-
     t0 = time.monotonic()
 
     # handle stdin if specified via lone '-'
@@ -1271,7 +1247,6 @@ def parse(
                 lnt.parse_string(
                     sys.stdin.read(),
                     "stdin",
-                    recurse=recurse,
                     config=lnt.config,
                     parse_statistics=parse_statistics,
                 ),
@@ -1281,7 +1256,6 @@ def parse(
             parsed_strings = list(
                 lnt.parse_path(
                     path=path,
-                    recurse=recurse,
                     parse_statistics=parse_statistics,
                 )
             )
@@ -1319,14 +1293,6 @@ def parse(
 
         # Dump the output to stdout or to file as appropriate.
         dump_file_payload(write_output, file_output)
-    if profiler:
-        pr.disable()
-        profiler_buffer = StringIO()
-        ps = pstats.Stats(pr, stream=profiler_buffer).sort_stats("cumulative")
-        ps.print_stats()
-        click.echo("==== profiler stats ====")
-        # Only print the first 50 lines of it
-        click.echo("\n".join(profiler_buffer.getvalue().split("\n")[:50]))
 
     if violations_count > 0 and not nofail:
         sys.exit(EXIT_FAIL)  # pragma: no cover
