@@ -6,34 +6,34 @@ except ImportError:  # pragma: no cover
     # fallback for python <=3.8
     from importlib_resources import files  # type: ignore
 
+import configparser
 import logging
 import os
 import os.path
-import configparser
 import sys
 from dataclasses import dataclass
-
-import pluggy
 from itertools import chain
+from pathlib import Path
 from typing import (
     TYPE_CHECKING,
+    Any,
+    Callable,
     Dict,
+    Iterable,
     Iterator,
     List,
+    Optional,
     Set,
     Tuple,
-    Any,
-    Optional,
     Type,
     Union,
-    Iterable,
-    Callable,
 )
-from pathlib import Path
-from sqlfluff.core.plugin.host import get_plugin_manager
-from sqlfluff.core.errors import SQLFluffUserError
 
 import appdirs
+import pluggy
+
+from sqlfluff.core.errors import SQLFluffUserError
+from sqlfluff.core.plugin.host import get_plugin_manager
 
 if TYPE_CHECKING:  # pragma: no cover
     from sqlfluff.core.templaters.base import RawTemplater
@@ -181,6 +181,10 @@ REMOVED_CONFIGS = [
         ("rules", "L003", "lint_templated_tokens"),
         "No longer used.",
     ),
+    _RemovedConfig(
+        ("core", "recurse"),
+        "Removed as unused in production and unnecessary for debugging.",
+    ),
 ]
 
 
@@ -943,9 +947,6 @@ class FluffConfig:
                 )
             else:
                 self._configs["core"][out_key] = []
-        # Configure Recursion
-        if self._configs["core"].get("recurse", 0) == 0:
-            self._configs["core"]["recurse"] = True
 
         # Dialect and Template selection.
         dialect: Optional[str] = self._configs["core"]["dialect"]
