@@ -1,14 +1,10 @@
 """Linter class and helper classes."""
 
-from sqlfluff.core.linter.common import (
-    RuleTuple,
-    ParsedString,
-    RenderedFile,
-)
-from sqlfluff.core.linter.noqa import NoQaDirective, IgnoreMask
+from sqlfluff.core.linter.common import ParsedString, RenderedFile, RuleTuple
 from sqlfluff.core.linter.linted_file import LintedFile
-from sqlfluff.core.linter.linting_result import LintingResult
 from sqlfluff.core.linter.linter import Linter
+from sqlfluff.core.linter.linting_result import LintingResult
+from sqlfluff.core.linter.noqa import IgnoreMask, NoQaDirective
 
 __all__ = (
     "RuleTuple",
