@@ -1,14 +1,14 @@
 """Defined the `match_wrapper` which adds validation and logging to match methods."""
 
-from typing import Any, Callable, Tuple, TYPE_CHECKING
+from typing import TYPE_CHECKING, Any, Callable, Tuple
 
+from sqlfluff.core.parser.helpers import join_segments_raw_curtailed
 from sqlfluff.core.parser.match_logging import ParseMatchLogObject
 from sqlfluff.core.parser.match_result import MatchResult
-from sqlfluff.core.parser.helpers import join_segments_raw_curtailed
 
 if TYPE_CHECKING:  # pragma: no cover
-    from sqlfluff.core.parser.context import ParseContext
     from sqlfluff.core.parser import BaseSegment
+    from sqlfluff.core.parser.context import ParseContext
 
 
 MatchFuncType = Callable[[Any, Tuple["BaseSegment", ...], "ParseContext"], MatchResult]
