@@ -1,15 +1,12 @@
 """Defines the Parser class."""
 
-from typing import Optional, Sequence, Type, TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional, Sequence, Type
 
-from sqlfluff.core.parser.context import ParseContext
 from sqlfluff.core.config import FluffConfig
+from sqlfluff.core.parser.context import ParseContext
 
 if TYPE_CHECKING:  # pragma: no cover
-    from sqlfluff.core.parser.segments import (
-        BaseSegment,
-        BaseFileSegment,
-    )
+    from sqlfluff.core.parser.segments import BaseFileSegment, BaseSegment
 
 
 class Parser:
@@ -27,7 +24,6 @@ class Parser:
     def parse(
         self,
         segments: Sequence["BaseSegment"],
-        recurse=True,
         fname: Optional[str] = None,
         parse_statistics: bool = False,
     ) -> Optional["BaseSegment"]:
@@ -38,13 +34,13 @@ class Parser:
             # api use cases.
             return None
         # Instantiate the root segment
-        root_segment = self.RootSegment(segments=segments, fname=fname)
+        root_segment = self.RootSegment(segments=tuple(segments), fname=fname)
         # Call .parse() on that segment
 
         # NOTE: This is the only time we use the parse context not in the
         # context of a context manager. That's because it's the initial
         # instantiation.
-        ctx = ParseContext.from_config(config=self.config, recurse=recurse)
+        ctx = ParseContext.from_config(config=self.config)
         parsed = root_segment.parse(parse_context=ctx)
 
         if parse_statistics:  # pragma: no cover
