@@ -16,49 +16,49 @@ missing.
 
 import bdb
 import copy
-from dataclasses import dataclass
 import fnmatch
-from itertools import chain
 import logging
 import pathlib
-import regex
 import re
+from collections import defaultdict, namedtuple
+from dataclasses import dataclass
+from itertools import chain
 from typing import (
     TYPE_CHECKING,
-    Sequence,
-    Sized,
-    cast,
-    Iterable,
-    Optional,
-    List,
-    Set,
-    Tuple,
-    Union,
     Any,
-    Dict,
-    Type,
     DefaultDict,
+    Dict,
+    Iterable,
     Iterator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Sized,
+    Tuple,
+    Type,
+    Union,
+    cast,
 )
-from collections import namedtuple, defaultdict
+
+import regex
 
 from sqlfluff.core.config import split_comma_separated_string
-
-from sqlfluff.core.linter import IgnoreMask
-from sqlfluff.core.parser import BaseSegment, PositionMarker, RawSegment
 from sqlfluff.core.dialects import Dialect
-from sqlfluff.core.errors import SQLLintError, SQLFluffUserError
+from sqlfluff.core.errors import SQLFluffUserError, SQLLintError
+from sqlfluff.core.parser import BaseSegment, PositionMarker, RawSegment
 from sqlfluff.core.parser.segments.base import SourceFix
+from sqlfluff.core.plugin.host import is_main_process, plugins_loaded
+from sqlfluff.core.rules.config_info import get_config_info
 from sqlfluff.core.rules.context import RuleContext
 from sqlfluff.core.rules.crawlers import BaseCrawler
-from sqlfluff.core.rules.config_info import get_config_info
-from sqlfluff.core.plugin.host import plugins_loaded, is_main_process
 from sqlfluff.core.templaters.base import RawFileSlice, TemplatedFile
 
 # Best solution for generic types on older python versions
 # https://github.com/python/typeshed/issues/7855
 if TYPE_CHECKING:  # pragma: no cover
     from sqlfluff.core.config import FluffConfig
+    from sqlfluff.core.linter import IgnoreMask
     from sqlfluff.core.plugin.hookspecs import PluginSpec
 
     _LoggerAdapter = logging.LoggerAdapter[logging.Logger]
@@ -197,14 +197,12 @@ class LintFix:
         self.anchor = anchor
         self.edit: Optional[List[BaseSegment]] = None
         if edit is not None:
-            # Coerce edit iterable to list
-            edit = list(edit)
             # Copy all the elements of edit to stop contamination.
             # We're about to start stripping the position markers
             # off some of the elements and we don't want to end up
             # stripping the positions of the original elements of
             # the parsed structure.
-            self.edit = copy.deepcopy(edit)
+            self.edit = [s.copy() for s in edit]
             # Check that any edits don't have a position marker set.
             # We should rely on realignment to make position markers.
             # Strip position markers of anything enriched, otherwise things can get
@@ -831,7 +829,7 @@ class BaseRule(metaclass=RuleMetaclass):
         dialect: Dialect,
         fix: bool,
         templated_file: Optional["TemplatedFile"],
-        ignore_mask: Optional[IgnoreMask],
+        ignore_mask: Optional["IgnoreMask"],
         fname: Optional[str],
         config: "FluffConfig",
     ) -> Tuple[
@@ -954,7 +952,7 @@ class BaseRule(metaclass=RuleMetaclass):
         self,
         res: LintResult,
         templated_file: Optional[TemplatedFile],
-        ignore_mask: Optional[IgnoreMask],
+        ignore_mask: Optional["IgnoreMask"],
         new_lerrs: List[SQLLintError],
         new_fixes: List[LintFix],
         root: BaseSegment,
