@@ -1,14 +1,13 @@
 """Templater Code."""
 
-from sqlfluff.core.templaters.base import TemplatedFile
+from typing import Iterator, Type
 
 # Although these shouldn't usually be instantiated from here
 # we import them to make sure they get registered.
-from sqlfluff.core.templaters.base import RawTemplater, RawFileSlice
+from sqlfluff.core.templaters.base import RawFileSlice, RawTemplater, TemplatedFile
 from sqlfluff.core.templaters.jinja import JinjaTemplater
-from sqlfluff.core.templaters.python import PythonTemplater
 from sqlfluff.core.templaters.placeholder import PlaceholderTemplater
-from typing import Iterator, Type
+from sqlfluff.core.templaters.python import PythonTemplater
 
 
 def core_templaters() -> Iterator[Type[RawTemplater]]:
