@@ -46,7 +46,7 @@ athena_dialect.insert_lexer_matchers(
     before="like_operator",
 )
 
-athena_dialect.sets("angle_bracket_pairs").update(
+athena_dialect.bracket_sets("angle_bracket_pairs").update(
     [
         ("angle", "StartAngleBracketSegment", "EndAngleBracketSegment", False),
     ]
@@ -673,13 +673,13 @@ class GroupByClauseSegment(ansi.GroupByClauseSegment):
                 Ref("NumericLiteralSegment"),  # Can `GROUP BY 1`
                 Ref("ExpressionSegment"),  # Can `GROUP BY coalesce(col, 1)`
             ),
-            terminator=OneOf(
+            terminators=[
                 Sequence("ORDER", "BY"),
                 "LIMIT",
                 "OFFSET",
                 "HAVING",
                 Ref("SetOperatorSegment"),
-            ),
+            ],
         ),
         Dedent,
     )
