@@ -6,10 +6,7 @@ from typing import cast, List, Set
 from sqlfluff.core.dialects.base import Dialect
 from sqlfluff.core.parser.segments import BaseSegment
 from sqlfluff.utils.analysis.select import get_select_statement_info
-from sqlfluff.utils.analysis.select_crawler import (
-    Query as SelectCrawlerQuery,
-    SelectCrawler,
-)
+from sqlfluff.utils.analysis.query import Query
 from sqlfluff.core.rules import (
     BaseRule,
     LintFix,
@@ -23,8 +20,8 @@ from sqlfluff.core.dialects.common import AliasInfo
 
 
 @dataclass
-class AL05Query(SelectCrawlerQuery):
-    """SelectCrawler Query with custom AL05 info."""
+class AL05Query(Query):
+    """Query subclass with custom AL05 info."""
 
     aliases: List[AliasInfo] = field(default_factory=list)
     tbl_refs: Set[str] = field(default_factory=set)
@@ -80,8 +77,7 @@ class Rule_AL05(BaseRule):
 
         # Analyze the SELECT.
         alias: AliasInfo
-        crawler = SelectCrawler(context.segment, context.dialect, query_class=AL05Query)
-        query: AL05Query = cast(AL05Query, crawler.query_tree)
+        query = AL05Query.from_segment(context.segment, dialect=context.dialect)
         self._analyze_table_aliases(query, context.dialect)
 
         if context.dialect.name == "redshift":
