@@ -1,11 +1,10 @@
 """Implementation of Rule AM04."""
 from typing import Optional, Tuple
 
-from sqlfluff.utils.analysis.select_crawler import Query, SelectCrawler
+from sqlfluff.utils.analysis.query import Query
 from sqlfluff.core.parser import BaseSegment
 from sqlfluff.core.rules import BaseRule, LintResult, RuleContext
 from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
-from sqlfluff.utils.functional import sp, FunctionalContext
 
 
 _START_TYPES = ["select_statement", "set_expression", "with_compound_statement"]
@@ -69,12 +68,13 @@ class Rule_AM04(BaseRule):
     name = "ambiguous.column_count"
     aliases = ("L044",)
     groups: Tuple[str, ...] = ("all", "ambiguous")
-    crawl_behaviour = SegmentSeekerCrawler(set(_START_TYPES))
+    # Only evaluate the outermost query.
+    crawl_behaviour = SegmentSeekerCrawler(set(_START_TYPES), allow_recurse=False)
 
     def _handle_alias(self, selectable, alias_info, query) -> None:
-        select_info_target = SelectCrawler.get(
-            query, alias_info.from_expression_element
-        )[0]
+        select_info_target = next(
+            query.crawl_sources(alias_info.from_expression_element, True)
+        )
         if isinstance(select_info_target, str):
             # It's an alias to an external table whose
             # number of columns could vary without our
@@ -126,10 +126,7 @@ class Rule_AM04(BaseRule):
                 else:
                     # No table was specified with the wildcard. Assume we're
                     # querying from a nested select in FROM.
-                    query_list = SelectCrawler.get(
-                        query, query.selectables[0].selectable
-                    )
-                    for o in query_list:
+                    for o in query.crawl_sources(query.selectables[0].selectable, True):
                         if isinstance(o, Query):
                             self._analyze_result_columns(o)
                             return None
@@ -141,13 +138,11 @@ class Rule_AM04(BaseRule):
 
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
         """Outermost query should produce known number of columns."""
-        if not FunctionalContext(context).parent_stack.any(sp.is_type(*_START_TYPES)):
-            crawler = SelectCrawler(context.segment, context.dialect)
+        query: Query = Query.from_segment(context.segment, context.dialect)
 
+        try:
             # Begin analysis at the outer query.
-            if crawler.query_tree:
-                try:
-                    self._analyze_result_columns(crawler.query_tree)
-                except RuleFailure as e:
-                    return LintResult(anchor=e.anchor)
-        return None
+            self._analyze_result_columns(query)
+            return None
+        except RuleFailure as e:
+            return LintResult(anchor=e.anchor)
