@@ -87,7 +87,11 @@ class Rule_AM06(BaseRule):
     crawl_behaviour = SegmentSeekerCrawler(
         {"groupby_clause", "orderby_clause", "grouping_expression_list"}
     )
-    _ignore_types: List[str] = ["withingroup_clause", "window_specification"]
+    _ignore_types: List[str] = [
+        "withingroup_clause",
+        "window_specification",
+        "aggregate_order_by",
+    ]
 
     def _eval(self, context: RuleContext) -> Optional[LintResult]:
         """Inconsistent column references in GROUP BY/ORDER BY clauses."""
