@@ -20,6 +20,7 @@ def get_rules() -> List[Type[BaseRule]]:
     from sqlfluff.rules.structure.ST06 import Rule_ST06
     from sqlfluff.rules.structure.ST07 import Rule_ST07
     from sqlfluff.rules.structure.ST08 import Rule_ST08
+    from sqlfluff.rules.structure.ST09 import Rule_ST09
 
     return [
         Rule_ST01,
@@ -30,4 +31,5 @@ def get_rules() -> List[Type[BaseRule]]:
         Rule_ST06,
         Rule_ST07,
         Rule_ST08,
+        Rule_ST09,
     ]
