@@ -1,8 +1,11 @@
 """Basic code analysis tools for SELECT statements."""
-from typing import List, NamedTuple, Optional
+from typing import List, NamedTuple, Optional, cast
 
 from sqlfluff.core.dialects.base import Dialect
-from sqlfluff.dialects.dialect_ansi import ObjectReferenceSegment
+from sqlfluff.dialects.dialect_ansi import (
+    ObjectReferenceSegment,
+    SelectClauseElementSegment,
+)
 from sqlfluff.core.dialects.common import AliasInfo, ColumnAliasInfo
 from sqlfluff.core.parser.segments.base import BaseSegment
 
@@ -14,11 +17,20 @@ class SelectStatementColumnsAndTables(NamedTuple):
     table_aliases: List[AliasInfo]
     standalone_aliases: List[str]  # value table function aliases
     reference_buffer: List[ObjectReferenceSegment]
-    select_targets: List[BaseSegment]
+    select_targets: List[SelectClauseElementSegment]
     col_aliases: List[ColumnAliasInfo]
     using_cols: List[str]
 
 
+def _get_object_references(segment: BaseSegment) -> List[ObjectReferenceSegment]:
+    return list(
+        cast(ObjectReferenceSegment, _seg)
+        for _seg in segment.recursive_crawl(
+            "object_reference", no_recursive_seg_type="select_statement"
+        )
+    )
+
+
 def get_select_statement_info(
     segment: BaseSegment, dialect: Optional[Dialect], early_exit: bool = True
 ) -> Optional[SelectStatementColumnsAndTables]:
@@ -34,7 +46,9 @@ def get_select_statement_info(
     # Sometimes there is no select clause (e.g. "SELECT *" is a select_clause_element)
     if not sc:
         return None
-    reference_buffer = list(sc.recursive_crawl("object_reference"))
+    # NOTE: In this first crawl, don't crawl inside any sub-selects, that's very
+    # important for both isolation and performance reasons.
+    reference_buffer = _get_object_references(sc)
     for potential_clause in (
         "where_clause",
         "groupby_clause",
@@ -44,22 +58,28 @@ def get_select_statement_info(
     ):
         clause = segment.get_child(potential_clause)
         if clause:
-            reference_buffer += list(clause.recursive_crawl("object_reference"))
+            reference_buffer += _get_object_references(clause)
 
     # Get all select targets.
-    select_targets = segment.get_child("select_clause").get_children(
-        "select_clause_element"
+    _select_clause = segment.get_child("select_clause")
+    assert _select_clause, "Select statement found without select clause."
+    select_targets = cast(
+        List[SelectClauseElementSegment],
+        _select_clause.get_children("select_clause_element"),
     )
 
-    # Get all column aliases
-    col_aliases = [s.get_alias() for s in select_targets if s.get_alias() is not None]
+    # Get all column aliases. NOTE: In two steps so mypy can follow.
+    _pre_aliases = [s.get_alias() for s in select_targets]
+    col_aliases = [_alias for _alias in _pre_aliases if _alias is not None]
 
     # Get any columns referred to in a using clause, and extract anything
     # from ON clauses.
     using_cols = []
     fc = segment.get_child("from_clause")
     if fc:
-        for join_clause in fc.recursive_crawl("join_clause"):
+        for join_clause in fc.recursive_crawl(
+            "join_clause", no_recursive_seg_type="select_statement"
+        ):
             seen_using = False
             for seg in join_clause.iter_segments():
                 if seg.is_type("keyword") and seg.raw_upper == "USING":
@@ -68,25 +88,13 @@ def get_select_statement_info(
                     for on_seg in seg.segments:
                         if on_seg.is_type("bracketed", "expression"):
                             # Deal with expressions
-                            reference_buffer += list(
-                                seg.recursive_crawl("object_reference")
-                            )
+                            reference_buffer += _get_object_references(seg)
                 elif seen_using and seg.is_type("bracketed"):
                     for subseg in seg.segments:
                         if subseg.is_type("identifier"):
                             using_cols.append(subseg.raw)
                     seen_using = False
 
-    # PURGE any references which are in nested select statements
-    for ref in reference_buffer.copy():
-        ref_path = segment.path_to(ref)
-        # is it in a subselect? i.e. a select which isn't this one.
-        if ref_path and any(
-            ps.segment.is_type("select_statement") and ps.segment is not segment
-            for ps in ref_path
-        ):
-            reference_buffer.remove(ref)
-
     return SelectStatementColumnsAndTables(
         select_statement=segment,
         table_aliases=table_aliases or [],
@@ -197,13 +205,14 @@ def _get_lambda_argument_columns(
             assert len(argument_segments) == 1
             child_segment = argument_segments[0]
 
-            if (
-                child_segment.is_type("bracketed")
-                and child_segment.get_child("start_bracket").raw == "("
-            ):
-                bracketed_arguments = child_segment.get_children("column_reference")
-                raw_arguments = [argument.raw for argument in bracketed_arguments]
-                lambda_argument_columns += raw_arguments
+            if child_segment.is_type("bracketed"):
+                start_bracket = child_segment.get_child("start_bracket")
+                # There will be a start bracket if it's bracketed.
+                assert start_bracket
+                if start_bracket.raw == "(":
+                    bracketed_arguments = child_segment.get_children("column_reference")
+                    raw_arguments = [argument.raw for argument in bracketed_arguments]
+                    lambda_argument_columns += raw_arguments
 
             elif child_segment.is_type("column_reference"):
                 lambda_argument_columns.append(child_segment.raw)
