@@ -19,7 +19,7 @@ from sqlfluff.core.templaters.base import TemplatedFile
 def raw_is(*raws: str) -> Callable[[BaseSegment], bool]:  # pragma: no cover
     """Returns a function that determines if segment matches one of the raw inputs."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return segment.raw in raws
 
     return _
@@ -28,7 +28,7 @@ def raw_is(*raws: str) -> Callable[[BaseSegment], bool]:  # pragma: no cover
 def raw_upper_is(*raws: str) -> Callable[[BaseSegment], bool]:
     """Returns a function that determines if segment matches one of the raw inputs."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return segment.raw_upper in raws
 
     return _
@@ -37,13 +37,13 @@ def raw_upper_is(*raws: str) -> Callable[[BaseSegment], bool]:
 def is_type(*seg_type: str) -> Callable[[BaseSegment], bool]:
     """Returns a function that determines if segment is one of the types."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return segment.is_type(*seg_type)
 
     return _
 
 
-def is_keyword(*keyword_name) -> Callable[[BaseSegment], bool]:
+def is_keyword(*keyword_name: str) -> Callable[[BaseSegment], bool]:
     """Returns a function that determines if it's a matching keyword."""
     return and_(
         is_type("keyword"), raw_upper_is(*[raw.upper() for raw in keyword_name])
@@ -125,7 +125,7 @@ def get_type() -> Callable[[BaseSegment], str]:
 def and_(*functions: Callable[[BaseSegment], bool]) -> Callable[[BaseSegment], bool]:
     """Returns a function that computes the functions and-ed together."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return all(function(segment) for function in functions)
 
     return _
@@ -134,7 +134,7 @@ def and_(*functions: Callable[[BaseSegment], bool]) -> Callable[[BaseSegment], b
 def or_(*functions: Callable[[BaseSegment], bool]) -> Callable[[BaseSegment], bool]:
     """Returns a function that computes the functions or-ed together."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return any(function(segment) for function in functions)
 
     return _
@@ -143,7 +143,7 @@ def or_(*functions: Callable[[BaseSegment], bool]) -> Callable[[BaseSegment], bo
 def not_(fn: Callable[[BaseSegment], bool]) -> Callable[[BaseSegment], bool]:
     """Returns a function that computes: not fn()."""
 
-    def _(segment: BaseSegment):
+    def _(segment: BaseSegment) -> bool:
         return not fn(segment)
 
     return _
